<template>
  <DatePicker
    v-model="modelValue"
    :columns="range ? 2 : 1"
    :first-day-of-week="2"
    :is-required="required"
    :max-date="maxDate"
    :min-date="minDate"
    :mode="time ? 'dateTime' : undefined"
    :model-modifiers="{ range: range }"
    :name="name"
    :popover="{ visibility: 'focus', placement: 'bottom-end' }"
    :timezone="Intl.DateTimeFormat().resolvedOptions().timeZone"
    color="primary"
    hide-time-header
    is24hr
  >
    <template #default="{ inputValue, togglePopover, updateValue }">
      <div class="group relative rounded-md bg-white">
        <div class="pointer-events-none absolute inset-y-0 left-0 flex items-center pl-3">
          <CalendarRegularIcon
            aria-hidden="true"
            class="h-4 w-4 fill-gray-400"
          />
        </div>
        <button
          :id="id"
          :class="classes"
          class="block h-9 w-full cursor-pointer rounded-md border-0 px-9 py-1.5 text-left text-gray-900 ring-1 ring-inset focus:outline-none focus:ring-2 focus:ring-primary-500 sm:text-sm sm:leading-6"
          type="button"
          @click="togglePopover"
        >
          <template v-if="range">
            <span v-if="inputValue.start || inputValue.end">{{ inputValue.start }} - {{ inputValue.end }}</span>
            <span v-else />
          </template>
          <span v-else>{{ inputValue }}</span>
        </button>
        <div
          v-if="!required"
          class="absolute right-0 top-0 flex h-9"
        >
          <button
            class="my-auto px-2 py-1"
            tabindex="-1"
            type="button"
            @click.prevent="updateValue"
          >
            <XMarkRegularIcon class="h-4 w-4 fill-red-500" />
          </button>
        </div>
      </div>
    </template>
  </DatePicker>
</template>

<script lang="ts" setup>
import type { ComputedRef, ModelRef } from "vue";
import { DatePicker } from "v-calendar";
import XMarkRegularIcon from "@/svg/icons/xmark-regular.svg?component";
import CalendarRegularIcon from "@/svg/icons/calendar-regular.svg?component";
import type { DatePickerModel, DateRange, SimpleDate } from "@/js/contexts/form/models/DatePicker";

defineProps<{
  name: string;
  required?: boolean;
  maxDate?: Date;
  minDate?: Date;
  range?: boolean;
  time?: boolean;
}>();

const modelValue = defineModel<DatePickerModel | undefined>({
  get: (value: DatePickerModel | undefined) => {
    if (!value) {
      return value;
    }
    if (typeof value === "object") {
      const range = value as DateRange;
      return { start: range?.start ? new Date(range.start) : undefined, end: range?.end ? new Date(range.end) : undefined };
    }
    const date = value as SimpleDate;
    return date ? new Date(date) : date;
  },
}) as ModelRef<DatePickerModel>;

const id = inject<string | undefined>("id", undefined);
const errors = inject<ComputedRef<boolean> | undefined>("errors", undefined);

const classes = computed(() => ({
  "ring-gray-900/10": !errors?.value,
  "text-red-600 ring-red-500 placeholder:text-red-400 focus:ring-red-600": errors?.value,
}));
</script>

<style>
@import "v-calendar/dist/style.css";

.vc-primary {
  --vc-accent-50: rgb(var(--color-primary-50));
  --vc-accent-100: rgb(var(--color-primary-100));
  --vc-accent-200: rgb(var(--color-primary-200));
  --vc-accent-300: rgb(var(--color-primary-300));
  --vc-accent-400: rgb(var(--color-primary-400));
  --vc-accent-500: rgb(var(--color-primary-500));
  --vc-accent-600: rgb(var(--color-primary-600));
  --vc-accent-700: rgb(var(--color-primary-700));
  --vc-accent-800: rgb(var(--color-primary-800));
  --vc-accent-900: rgb(var(--color-primary-900));
  --vc-accent-950: rgb(var(--color-primary-950));
}
</style>
